# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4
#
# Savon - sysadmin versioning
# Copyright (C) 2006 Thomas Vander Stichele <thomas at apestaart dot org>

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.

"""
'resolve' command.
"""

from savon import actor, state
from savon.util import log, util

class Diff(util.LogCommand):
    usage = "diff [live paths]"
    description = "show the diffs that need resolving"

    def do(self, args):
        self.debug('diff %r' % args)
        actor = self.getRootCommand().actor
        livePaths = util.getAbsolutePaths(args)
        self.debug('diff on livePaths %r' % livePaths)
        outputs = actor.resolveDiff(livePaths)
        for output in outputs:
            self.stdout.write(output + '\n')

class Edit(util.LogCommand):
    usage = "edit [live paths]"
    description = "resolve checkout conflicts by editing diffs"

    def addOptions(self):
        self.debug('adding option')
        self.parser.add_option('-e', '--editor',
            action="store", dest="editor",
            help="editor to use to resolve conflicts")

    def handleOptions(self, options):
        self.debug('handling options %r' % options)
        self.editor = options.editor

    def do(self, args):
        actor = self.getRootCommand().actor
        livePaths = util.getAbsolutePaths(args)
        editor = "meld"
        if self.editor:
            editor = self.editor
        self.debug('edit using %r on livePaths %r' % (editor, livePaths))
        actor.resolveEdit(editor, livePaths)

class List(util.LogCommand):
    usage = "list [live paths]"
    description = "show list of remaining conflicts"

    def do(self, args):
        actor = self.getRootCommand().actor
        conflicts = actor.getConflictsForLivePaths(args)

        if not conflicts:
            self.stdout.write('No conflicts.\n')
        else:
            self.stdout.write('The following paths are in conflict:\n')
            for livePath in conflicts:
                self.stdout.write('  %s\n' % livePath)

class Live(util.LogCommand):
    usage = "live [live paths]"
    description = "resolve checkout conflicts by preferring live version"

    def do(self, args):
        actor = self.getRootCommand().actor
        livePaths = util.getAbsolutePaths(args)
        actor.resolveLive(livePaths)

class Repository(util.LogCommand):
    usage = "repository [live paths]"
    description = "resolve checkout conflicts by preferring repository version"
    aliases = ["repo", ]

    def do(self, args):
        actor = self.getRootCommand().actor
        livePaths = util.getAbsolutePaths(args)
        actor.resolveRepository(livePaths)

class Resolve(util.LogCommand):
    usage = "resolve %command"
    description = "resolve checkout conflicts"
    subCommandClasses = (Diff, Edit, List, Live, Repository)
